########################################################################
# py
#
# Functions library for the Mallards Scoreboard.
#
#    Copyright (C) 2015 Anders Lind, Erding Mallards e.V.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Import all modules referenced by this module
import os
import time
import datetime

import sbvar

###############################################################################
# get_counter_value
#
# Returns a text string showing the value of the given global counter
#
# Arguments: 
#   counter [string]
#
# Returns: 
#   cval [string]
def get_counter_value(counter):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'get_counter_value(counter), arg:', sbvar.col_fs, counter, sbvar.col_n)

    if counter == 'G1':
        cval = sbvar.counter_G1
    elif counter == 'H1':
        cval = sbvar.counter_H1
    elif counter == 'G2':
        cval = sbvar.counter_G2
    elif counter == 'H2':
        cval = sbvar.counter_H2
    elif counter == 'G3':
        cval = sbvar.counter_G3
    elif counter == 'H3':
        cval = sbvar.counter_H3
    elif counter == 'G4':
        cval = sbvar.counter_G4
    elif counter == 'H4':
        cval = sbvar.counter_H4
    elif counter == 'G5':
        cval = sbvar.counter_G5
    elif counter == 'H5':
        cval = sbvar.counter_H5
    elif counter == 'G6':
        cval = sbvar.counter_G6
    elif counter == 'H6':
        cval = sbvar.counter_H6
    elif counter == 'G7':
        cval = sbvar.counter_G7
    elif counter == 'H7':
        cval = sbvar.counter_H7
    elif counter == 'G8':
        cval = sbvar.counter_G8
    elif counter == 'H8':
        cval = sbvar.counter_H8
    elif counter == 'G9':
        cval = sbvar.counter_G9
    elif counter == 'H9':
        cval = sbvar.counter_H9
    elif counter == 'GX':
        cval = sbvar.counter_GX
    elif counter == 'HX':
        cval = sbvar.counter_HX
    elif counter == 'GR':
        cval = sbvar.counter_GR
    elif counter == 'HR':
        cval = sbvar.counter_HR
    elif counter == 'GH':
        cval = sbvar.counter_GH
    elif counter == 'HH':
        cval = sbvar.counter_HH
    elif counter == 'GE':
        cval = sbvar.counter_GE
    elif counter == 'HE':
        cval = sbvar.counter_HE
    elif counter == 'OUTS':
        cval = sbvar.counter_OUTS
    elif counter == 'BALLS':
        cval = sbvar.counter_BALLS
    elif counter == 'STRIKES':
        cval = sbvar.counter_STRIKES
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, '(get_counter_value): Invalid counter given:',sbvar.col_er, counter, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'get_counter_value(counter), return:', sbvar.col_fe, cval, sbvar.col_n, sbvar.col_t, exec_time, sbvar.col_n)
    
    return cval



########################################################################
# get_current_time
#
# Gets the current system time.
#
# Arguments: none
#
# Returns:
#   hours_x10 [integer]
#   hours_x1 [integer]
#   minutes_x10 [integer]
#   minutes_x1 [integer]
def get_current_time():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'get_current_time()')

    hours = time.strftime("%H", time.localtime() )
    minutes = time.strftime("%M", time.localtime() )
    hours_x1 = int(hours) % 10
    hours_x10 = int(hours) // 10
    minutes_x1 = int(minutes) % 10
    minutes_x10 = int(minutes) // 10

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, ' get_current_time(), return:', sbvar.col_fe, hours_x10, hours_x1, minutes_x10, minutes_x1, sbvar.col_n, sbvar.col_t, exec_time, sbvar.col_n)
  
    return (hours_x10, hours_x1, minutes_x10, minutes_x1)



###############################################################################
# get_inning_label
#
# Returns a text string showing the current inning
#
# Arguments: none
#
# Returns: 
#   label [string]
def get_inning_label():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'get_inning_label()')

    if sbvar.counter_INNING == 0:
        label = 'Pre-Game'
    elif sbvar.counter_INNING == 1:
        label = 'Top 1st'
    elif sbvar.counter_INNING == 2:
        label = 'Bottom 1st'
    elif sbvar.counter_INNING == 3:
        label = 'Top 2nd'
    elif sbvar.counter_INNING == 4:
        label = 'Bottom 2nd'
    elif sbvar.counter_INNING == 5:
        label = 'Top 3rd'
    elif sbvar.counter_INNING == 6:
        label = 'Bottom 3rd'
    elif sbvar.counter_INNING == 7:
        label = 'Top 4th'
    elif sbvar.counter_INNING == 8:
        label = 'Bottom 4th'
    elif sbvar.counter_INNING == 9:
        label = 'Top 5th'
    elif sbvar.counter_INNING == 10:
        label = 'Bottom 5th'
    elif sbvar.counter_INNING == 11:
        label = 'Top 6th'
    elif sbvar.counter_INNING == 12:
        label = 'Bottom 6th'
    elif sbvar.counter_INNING == 13:
        label = 'Top 7th'
    elif sbvar.counter_INNING == 14:
        label = 'Bottom 7th'
    elif sbvar.counter_INNING == 15:
        label = 'Top 8th'
    elif sbvar.counter_INNING == 16:
        label = 'Bottom 8th'
    elif sbvar.counter_INNING == 17:
        label = 'Top 9th'
    elif sbvar.counter_INNING == 18:
        label = 'Bottom 9th'
    elif sbvar.counter_INNING == 19:
        label = 'Top 10th'
    elif sbvar.counter_INNING == 20:
        label = 'Bottom 10th'
    elif sbvar.counter_INNING == 21:
        label = 'Top 11th'
    elif sbvar.counter_INNING == 22:
        label = 'Bottom 11th'
    elif sbvar.counter_INNING == 23:
        label = 'Top 12th'
    elif sbvar.counter_INNING == 24:
        label = 'Bottom 12th'
    elif sbvar.counter_INNING == 25:
        label = 'Top 13th'
    elif sbvar.counter_INNING == 26:
        label = 'Bottom 13th'
    elif sbvar.counter_INNING == 27:
        label = 'Top 14th'
    elif sbvar.counter_INNING == 28:
        label = 'Bottom 14th'
    elif sbvar.counter_INNING == 29:
        label = 'Top 15th'
    elif sbvar.counter_INNING == 30:
        label = 'Bottom 15th'
    elif sbvar.counter_INNING == 31:
        label = 'Top 16th'
    elif sbvar.counter_INNING == 32:
        label = 'Bottom 16th'
    elif sbvar.counter_INNING == 33:
        label = 'Top 17th'
    elif sbvar.counter_INNING == 34:
        label = 'Bottom 17th'
    elif sbvar.counter_INNING == 35:
        label = 'Top 18th'
    elif sbvar.counter_INNING == 36:
        label = 'Bottom 18th'
    elif sbvar.counter_INNING == 37:
        label = 'Top 19th'
    elif sbvar.counter_INNING == 38:
        label = 'Bottom 19th'
    elif sbvar.counter_INNING == 39:
        label = 'Top 20th'
    elif sbvar.counter_INNING == 40:
        label = 'Bottom 20th'
    elif sbvar.counter_INNING == 41:
        label = 'Top 21st'
    elif sbvar.counter_INNING == 42:
        label = 'Bottom 21st'
    elif sbvar.counter_INNING == 43:
        label = 'Top 22nd'
    elif sbvar.counter_INNING == 44:
        label = 'Bottom 22nd'
    elif sbvar.counter_INNING == 45:
        label = 'Top 23rd'
    elif sbvar.counter_INNING == 46:
        label = 'Bottom 23rd'
    elif sbvar.counter_INNING == 47:
        label = 'Top 24th'
    elif sbvar.counter_INNING == 48:
        label = 'Bottom 24th'
    elif sbvar.counter_INNING == 49:
        label = 'Top 25th'
    elif sbvar.counter_INNING == 50:
        label = 'Bottom 25th'
    elif sbvar.counter_INNING == 51:
        label = 'Top 26th'
    elif sbvar.counter_INNING == 52:
        label = 'Bottom 26th'
    elif sbvar.counter_INNING == 53:
        label = 'Top 27th'
    elif sbvar.counter_INNING == 54:
        label = 'Bottom 27th'
    elif sbvar.counter_INNING == 55:
        label = 'Top 28th'
    elif sbvar.counter_INNING == 56:
        label = 'Bottom 28th'
    elif sbvar.counter_INNING == 57:
        label = 'Top 29th'
    elif sbvar.counter_INNING == 58:
        label = 'Bottom 29th'
    elif sbvar.counter_INNING == 59:
        label = 'Top 30th'
    elif sbvar.counter_INNING == 60:
        label = 'Bottom 30th'
    elif sbvar.counter_INNING == 61:
        label = 'Top 31st'
    elif sbvar.counter_INNING == 62:
        label = 'Bottom 31st'
    elif sbvar.counter_INNING == 63:
        label = 'Top 32nd'
    elif sbvar.counter_INNING == 64:
        label = 'Bottom 32nd'
    elif sbvar.counter_INNING == 65:
        label = 'Top 33rd'
    elif sbvar.counter_INNING == 66:
        label = 'Bottom 33dr'
    elif sbvar.counter_INNING == 67:
        label = 'Top 34th'
    elif sbvar.counter_INNING == 68:
        label = 'Bottom 34th'
    elif sbvar.counter_INNING == 69:
        label = 'Top 35th'
    elif sbvar.counter_INNING == 70:
        label = 'Bottom 35th'
    elif sbvar.counter_INNING == 71:
        label = 'Top 36th'
    elif sbvar.counter_INNING == 72:
        label = 'Bottom 36th'
    elif sbvar.counter_INNING == 73:
        label = 'Top 37th'
    elif sbvar.counter_INNING == 74:
        label = 'Bottom 37th'
    elif sbvar.counter_INNING == 75:
        label = 'Top 38th'
    elif sbvar.counter_INNING == 76:
        label = 'Bottom 38th'
    elif sbvar.counter_INNING == 77:
        label = 'Top 39th'
    elif sbvar.counter_INNING == 78:
        label = 'Bottom 39th'
    else:
        pass

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'get_inning_label(), return:', sbvar.col_fe, label, sbvar.col_n, sbvar.col_t, exec_time, sbvar.col_n)

    return label



########################################################################
# get_number_code
#
# Returns the hexadecimal register value corresponding to the given 
# number to be displayed.
#
# Arguments: 
#   number [integer]
#
# Returns: 
#   hexvalue = character value [hexadecimal]
def get_number_code(number):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'get_number_code(number), arg:',sbvar.col_fs, number, sbvar.col_n)

    if number <= 0:
        hexvalue = 0x3F
    elif number == 1:
        hexvalue = 0x06
    elif number == 2:
        hexvalue = 0x5B
    elif number == 3:
        hexvalue = 0x4F
    elif number == 4:
        hexvalue = 0x66
    elif number == 5:
        hexvalue = 0x6D
    elif number == 6:
        hexvalue = 0x7D
    elif number == 7:
        hexvalue = 0x07
    elif number == 8:
        hexvalue = 0x7F
    elif number == 9:
        hexvalue = 0x6F
    elif number == 10:
        hexvalue = 0xBF
    elif number == 11:
        hexvalue = 0x86
    elif number == 12:
        hexvalue = 0xDB
    elif number == 13:
        hexvalue = 0xCF
    elif number == 14:
        hexvalue = 0xE6
    elif number == 15:
        hexvalue = 0xED
    elif number == 16:
        hexvalue = 0xFD
    elif number == 17:
        hexvalue = 0x87
    elif number == 18:
        hexvalue = 0xFF
    elif number == 19:
        hexvalue = 0xEF
    elif number > 19:
        hexvalue = 0xF3
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'get_number_code(number): Invalid number provided:', sbvar.col_er, number, sbvar.col_n)
   
    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'get_number_code(number), return:',sbvar.col_fe, str(hexvalue), sbvar.col_n, sbvar.col_t, exec_time, sbvar.col_n)

    return str(hexvalue)



########################################################################
# get_segment_address
#
# Used to get the i2c address associated with the given scoreboard 
# segment.
#
# Input: 
#    segment [string]
#
# Output:
#   i2c_bus = i2c bus number (0|1) [integer]
#   i2c_address = i2c address (0x20|0x21|0x22|0x23|0x24|0x25|0x26|0x27) [hex]
#   i2c_register = i2c register/bank (0x14|0x15) [hex]

def get_segment_address(segment):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'get_segment_address(segment), arg:', sbvar.col_fs, segment, sbvar.col_n)

    if segment == "G1":
        i2c_bus = 0
        i2c_address = 0x20
        i2c_register = 0x14
    elif segment == "G2":
        i2c_bus = 0
        i2c_address = 0x20
        i2c_register = 0x15
    elif segment == "G3":
        i2c_bus = 0
        i2c_address = 0x21
        i2c_register = 0x14
    elif segment == "G4":
        i2c_bus = 0
        i2c_address = 0x21
        i2c_register = 0x15
    elif segment == "G5":
        i2c_bus = 0
        i2c_address = 0x22
        i2c_register = 0x14
    elif segment == "G6":
        i2c_bus = 0
        i2c_address = 0x22
        i2c_register = 0x15
    elif segment == "G7":
        i2c_bus = 0
        i2c_address = 0x23
        i2c_register = 0x14
    elif segment == "G8":
        i2c_bus = 0
        i2c_address = 0x23
        i2c_register = 0x15
    elif segment == "G9":
        i2c_bus = 0
        i2c_address = 0x24
        i2c_register = 0x14
    elif segment == "GX":
        i2c_bus = 0
        i2c_address = 0x24
        i2c_register = 0x15
    elif segment == "H1":
        i2c_bus = 1
        i2c_address = 0x20
        i2c_register = 0x14
    elif segment == "H2":
        i2c_bus = 1
        i2c_address = 0x20
        i2c_register = 0x15
    elif segment == "H3":
        i2c_bus = 1
        i2c_address = 0x21
        i2c_register = 0x14
    elif segment == "H4":
        i2c_bus = 1
        i2c_address = 0x21
        i2c_register = 0x15
    elif segment == "H5":
        i2c_bus = 1
        i2c_address = 0x22
        i2c_register = 0x14
    elif segment == "H6":
        i2c_bus = 1
        i2c_address = 0x22
        i2c_register = 0x15
    elif segment == "H7":
        i2c_bus = 1
        i2c_address = 0x23
        i2c_register = 0x14
    elif segment == "H8":
        i2c_bus = 1
        i2c_address = 0x23
        i2c_register = 0x15
    elif segment == "H9":
        i2c_bus = 1
        i2c_address = 0x24
        i2c_register = 0x14
    elif segment == "HX":
        i2c_bus = 1
        i2c_address = 0x24
        i2c_register = 0x15
    elif segment == "GE":
        i2c_bus = 1
        i2c_address = 0x27
        i2c_register = 0x14
    elif segment == "HE":
        i2c_bus = 1
        i2c_address = 0x27
        i2c_register = 0x15
    elif segment == "GRx1" or segment == "B2":
        i2c_bus = 0
        i2c_address = 0x25
        i2c_register = 0x15
    elif segment == "GRx10" or segment == "B1":
        i2c_bus = 0
        i2c_address = 0x25
        i2c_register = 0x14
    elif segment == "HRx1" or segment == "S2":
        i2c_bus = 1
        i2c_address = 0x25
        i2c_register = 0x15
    elif segment == "HRx10" or segment == "S1":
        i2c_bus = 1
        i2c_address = 0x25
        i2c_register = 0x14
    elif segment == "GHx1" or segment == "COLON":
        i2c_bus = 0
        i2c_address = 0x26
        i2c_register = 0x15
    elif segment == "GHx10" or segment == "B3":
        i2c_bus = 0
        i2c_address = 0x26
        i2c_register = 0x14
    elif segment == "HHx1" or segment == "O2":
        i2c_bus = 1
        i2c_address = 0x26
        i2c_register = 0x15
    elif segment == "HHx10" or segment == "O1":
        i2c_bus = 1
        i2c_address = 0x26
        i2c_register = 0x14
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'get_segment_address(segment): Invalid segment name:', sbvar.col_er, segment, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'get_segment_address(segment), return:', sbvar.col_fe, i2c_bus, i2c_address, i2c_register, sbvar.col_n, sbvar.col_t, exec_time, sbvar.col_n)
    
    return [i2c_bus, i2c_address, i2c_register]



########################################################################
# set_display_mode
#
# Configures the visibility flags for the different display modes.
# Refreshes the display.
#
# Arguments:
#   dispmode (0|1|2) [integer]
#
# Returns: nothing
#
#   0  Show clock
#   1  Show all
#   2  Hide hits, errors, balls and strikes

def set_display_mode(dispmode):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'set_display_mode(dispmode), arg:', sbvar.col_fs, dispmode, sbvar.col_n)

    if dispmode != 0 and dispmode != 1 and dispmode != 2 and dispmode != 3:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'set_display_mode(): illegal mode given:', sbvar.col_er, dispmode, sbvar.col_n)
        return None

    if dispmode == 0:
        ### Time dispmode ###
        sbvar.display_mode = 0
        sbvar.show_G1 = 0
        sbvar.show_G2 = 0
        sbvar.show_G3 = 0
        sbvar.show_G4 = 0
        sbvar.show_G5 = 0
        sbvar.show_G6 = 1
        sbvar.show_G7 = 1
        sbvar.show_G8 = 0
        sbvar.show_G9 = 1
        sbvar.show_GX = 1
        # Home inning flags:
        sbvar.show_H1 = 0
        sbvar.show_H2 = 0
        sbvar.show_H3 = 0
        sbvar.show_H4 = 0
        sbvar.show_H5 = 0
        sbvar.show_H6 = 0
        sbvar.show_H7 = 0
        sbvar.show_H8 = 0
        sbvar.show_H9 = 0
        sbvar.show_HX = 0
        # Error flags:
        sbvar.show_GE = 0
        sbvar.show_HE = 0
        # Runs flags:
        sbvar.show_GR = 0
        sbvar.show_HR = 0
        # Hits flags:
        sbvar.show_GH = 0
        sbvar.show_HH = 0
        # Outs, balls, strikes counters:
        sbvar.show_OUTS = 0
        sbvar.show_BALLS = 0
        sbvar.show_STRIKES = 0
        sbvar.show_COLON = 1
        all_segs = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                   'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX',\
                   'GRx1','GRx10','GHx1','GHx10','GE',\
                   'HRx1','HRx10','HHx1','HHx10','HE']
        for i in all_segs:
            show_one_character(i, 0x0)
        refresh_colon()
        show_time()
    elif dispmode == 1:
        ### Baseball dispmode ###
        sbvar.display_mode = 1
        update_inning_count('refresh')
        # Error flags:
        sbvar.show_GE = 1
        sbvar.show_HE = 1
        # Runs flags:
        sbvar.show_GR = 1
        sbvar.show_HR = 1
        # Hits flags:
        sbvar.show_GH = 1
        sbvar.show_HH = 1
        # Outs, balls, strikes counters:
        sbvar.show_OUTS = 1
        sbvar.show_BALLS = 1
        sbvar.show_STRIKES = 1
        sbvar.show_COLON = 0
        refresh_runs('G')
        refresh_runs('H')
        refresh_hits('G',0)
        refresh_hits('H',0)
        refresh_errors('GE',0)
        refresh_errors('HE',0)
        refresh_colon()
        refresh_outs()
        refresh_strikes()
        refresh_balls()
    elif dispmode == 2:
        ### Teeball dispmode ###
        sbvar.display_mode = 2
        update_inning_count('refresh')
        # Error flags:
        sbvar.show_GE = 0
        sbvar.show_HE = 0
        # Runs flags:
        sbvar.show_GR = 1
        sbvar.show_HR = 1
        # Hits flags:
        sbvar.show_GH = 0
        sbvar.show_HH = 0
        # Outs, balls, strikes counters:
        sbvar.show_OUTS = 1
        sbvar.show_BALLS = 0
        sbvar.show_STRIKES = 0
        sbvar.show_COLON = 0
        refresh_hits('G',0)
        refresh_hits('H',0)
        refresh_errors('GE',0)
        refresh_errors('HE',0)
        refresh_colon()
        refresh_outs()
        refresh_strikes()
        refresh_balls()
        refresh_runs('G')
        refresh_runs('H')
    elif dispmode == 3:
        # Welcome message
        all_segs = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                   'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX',\
                   'GRx10','GRx1','GHx10','GHx1','GE',\
                   'HRx10','HRx1','HHx10','HHx1','HE']
        for i in all_segs:
            show_one_character(i, 0x0)
        
        show_one_character('G1', 0x79) # G1: "E"
        show_one_character('G2', 0x50) # G2: "r"
        show_one_character('G3', 0x5E) # G3: "d"
        show_one_character('G4', 0x06) # G4: "I"
        show_one_character('G5', 0x54) # G5: "n"
        show_one_character('G6', 0x3D) # G6: "G"

        show_one_character('H1', 0x33) # H1: "M (left)"
        show_one_character('H2', 0x27) # H2: "M (right)"
        show_one_character('H3', 0x77) # H3: "A"
        show_one_character('H4', 0x38) # H4: "L"
        show_one_character('H5', 0x38) # H5: "L"
        show_one_character('H6', 0x77) # H6: "A"
        show_one_character('H7', 0x50) # H7: "r"
        show_one_character('H8', 0x5E) # H8: "d"
        show_one_character('H9', 0x6D) # H9: "s"

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'set_display_mode(dispmode)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# set_time
#
# Sets the  system time.
#
# Arguments:
#   time_string [string]
#
# Returns: nothing
#
# Format: sudo date -s "02:22"

def set_time(time_string):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'set_time(time_string), arg:', sbvar.col_fs, time_string, sbvar.col_n)

    new_string = time_string.replace('%20', '')
    call_string = 'sudo date -s ' + new_string
    os.system(call_string)

    if sbvar.display_mode == 0:
        set_display_mode(0)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'set_time(time_string)', sbvar.col_t, exec_time, sbvar.col_n)
  
    return None



###############################################################################
# show_one_character
#
# Function to display one given character on a specified segment.
#
# Arguments:
#   segment [string]
#   character [string]
#
# Returns: nothing
def show_one_character(segment, character):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'show_one_character(segment, character), arg:', sbvar.col_fs, segment, character, sbvar.col_n)
    
    i2c_bus, i2c_adr, i2c_reg = get_segment_address(segment)
    call_string = "i2cset -y" + " " + str(i2c_bus) + " " + \
        str(i2c_adr) + " " + str(i2c_reg) + " " + str(character)
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'show_one_character(segment, character)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



###############################################################################
# show_time
#
# Function to display the current time. Will check that the display_mode
# is set to 0 = time mode active.
#
# Arguments: none
#
# Returns: nothing
def show_time():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_wa, time.strftime("%H:%M:%S"), sbvar.col_n, 'show_time()')

    i2c_bus_h10, i2c_adr_h10, i2c_reg_h10 = \
        get_segment_address('G6')
    i2c_bus_h1, i2c_adr_h1, i2c_reg_h1 = \
        get_segment_address('G7')
    i2c_bus_m10, i2c_adr_m10, i2c_reg_m10 = \
        get_segment_address('G9')
    i2c_bus_m1, i2c_adr_m1, i2c_reg_m1 = \
        get_segment_address('GX')

    hours_x10, hours_x1, minutes_x10, minutes_x1 = \
        get_current_time()

    call_string = "i2cset -y" + " " + str(i2c_bus_h10) + " " + \
        str(i2c_adr_h10) + " " + str(i2c_reg_h10) + " " + \
        str(get_number_code(hours_x10))
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_h1) + " " + \
        str(i2c_adr_h1) + " " + str(i2c_reg_h1) + " " + \
        str(get_number_code(hours_x1))
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_m10) + " " + \
        str(i2c_adr_m10) + " " + str(i2c_reg_m10) + " " + \
        str(get_number_code(minutes_x10))
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_m1) + " " + \
        str(i2c_adr_m1) + " " + str(i2c_reg_m1) + " " + \
        str(get_number_code(minutes_x1))
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_wa, time.strftime("%H:%M:%S"), sbvar.col_n, 'show_time()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# show_inning
#
# Turns on the display variable for one inning
#
# Arguments:
#   inning [string]
#
# Returns: nothing
def show_inning(inning):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'show_inning(inning), arg:', sbvar.col_fs, inning, sbvar.col_n)

    if inning == 'G1':
        sbvar.show_G1 = 1
    elif inning == 'H1':
        sbvar.show_H1 = 1
    elif inning == 'G2':
        sbvar.show_G2 = 1
    elif inning == 'H2':
        sbvar.show_H2 = 1
    elif inning == 'G3':
        sbvar.show_G3 = 1
    elif inning == 'H3':
        sbvar.show_H3 = 1
    elif inning == 'G4':
        sbvar.show_G4 = 1
    elif inning == 'H4':
        sbvar.show_H4 = 1
    elif inning == 'G5':
        sbvar.show_G5 = 1
    elif inning == 'H5':
        sbvar.show_H5 = 1
    elif inning == 'G6':
        sbvar.show_G6 = 1
    elif inning == 'H6':
        sbvar.show_H6 = 1
    elif inning == 'G7':
        sbvar.show_G7 = 1
    elif inning == 'H7':
        sbvar.show_H7 = 1
    elif inning == 'G8':
        sbvar.show_G8 = 1
    elif inning == 'H8':
        sbvar.show_H8 = 1
    elif inning == 'G9':
        sbvar.show_G9 = 1
    elif inning == 'H9':
        sbvar.show_H9 = 1
    elif inning == 'GX':
        sbvar.show_GX = 1
    elif inning == 'HX':
        sbvar.show_HX = 1
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, '(show_inning): Illegal inning name given:', sbvar.col_er, inning, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'show_inning(inning)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# hide_inning
#
# Turns off the display variable for one inning
#
# Arguments:
#   inning [string]
#
# Returns: nothing
def hide_inning(inning):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'hide_inning(inning), arg:', sbvar.col_fs, inning, sbvar.col_n)

    if inning == 'G1':
        sbvar.show_G1 = 0
    elif inning == 'H1':
        sbvar.show_H1 = 0
    elif inning == 'G2':
        sbvar.show_G2 = 0
    elif inning == 'H2':
        sbvar.show_H2 = 0
    elif inning == 'G3':
        sbvar.show_G3 = 0
    elif inning == 'H3':
        sbvar.show_H3 = 0
    elif inning == 'G4':
        sbvar.show_G4 = 0
    elif inning == 'H4':
        sbvar.show_H4 = 0
    elif inning == 'G5':
        sbvar.show_G5 = 0
    elif inning == 'H5':
        sbvar.show_H5 = 0
    elif inning == 'G6':
        sbvar.show_G6 = 0
    elif inning == 'H6':
        sbvar.show_H6 = 0
    elif inning == 'G7':
        sbvar.show_G7 = 0
    elif inning == 'H7':
        sbvar.show_H7 = 0
    elif inning == 'G8':
        sbvar.show_G8 = 0
    elif inning == 'H8':
        sbvar.show_H8 = 0
    elif inning == 'G9':
        sbvar.show_G9 = 0
    elif inning == 'H9':
        sbvar.show_H9 = 0
    elif inning == 'GX':
        sbvar.show_GX = 0
    elif inning == 'HX':
        sbvar.show_HX = 0
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, '(hide_inning): Illegal inning name given:', sbvar.col_er, inning, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'hide_inning(inning)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# next_batter
#
# This function resets the BALLS and STRIKES counters and refreshes
# the display.
#
# Arguments: none
#
# Returns: nothing
#
def next_batter():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'next_batter()')

    if sbvar.display_mode == 1:   
        if sbvar.counter_INNING == 0:
            if sbvar.debug_mode != 0:
                print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'next_batter() - no update if in Pre-Game')
            return None
        sbvar.counter_STRIKES = 0
        sbvar.counter_BALLS = 0
        refresh_strikes()
        refresh_balls()
    else:
        print(sbvar.col_wa, '    WARNING:', sbvar.col_n, 'next_batter() - no update if not in Baseball mode')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'next_batter()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_balls
#
# Sends the corresponding i2c commands to turn the two out lamps on or 
# off depending on the value of the sbvar.counter_BALLS counter. Since 
# the lamps are implemented as bit 7 of the GRx1, GRx10 and GHx10 
# registers, a boolean operation is first done to not disrupt the digits.
#
# See comments in the refresh_outs() function for details.
#
# B1 => GRx10
# B2 => GRx1
# B3 => GHx10
#
# Arguments: none
#
# Returns: nothing
def refresh_balls():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_balls()')

    counter_GRx1 = sbvar.counter_GR % 10
    counter_GRx10 = sbvar.counter_GR // 10
    counter_GHx10 = sbvar.counter_GH // 10
    i2c_bus_GRx1, i2c_adr_GRx1, i2c_reg_GRx1 = \
        get_segment_address('GRx1')
    i2c_bus_GRx10, i2c_adr_GRx10, i2c_reg_GRx10 = \
        get_segment_address('GRx10')
    i2c_bus_GHx10, i2c_adr_GHx10, i2c_reg_GHx10 = \
        get_segment_address('GHx10')

    if sbvar.show_BALLS != 0:
        if sbvar.counter_BALLS == 0:
            dig_GRx1 = 0x7F & int(get_number_code(counter_GRx1))
            if counter_GRx10 == 0:
                dig_GRx10 = 0x0
            else:
                dig_GRx10 = 0x7F & int(get_number_code(counter_GRx10))
            if counter_GHx10 == 0:
                dig_GHx10 = 0x0
            else:
                dig_GHx10 = 0x7F & int(get_number_code(counter_GHx10))
        elif sbvar.counter_BALLS == 1:
            dig_GRx1 = 0x7F & int(get_number_code(counter_GRx1))
            if counter_GRx10 == 0:
                dig_GRx10 = 0x80
            else:
                dig_GRx10 = 0x80 | int(get_number_code(counter_GRx10))
            if counter_GHx10 == 0:
                dig_GHx10 = 0x0
            else:
                dig_GHx10 = 0x7F & int(get_number_code(counter_GHx10))
        elif sbvar.counter_BALLS == 2:
            dig_GRx1 = 0x80 | int(get_number_code(counter_GRx1))
            if counter_GRx10 == 0:
                dig_GRx10 = 0x80
            else:
                dig_GRx10 = 0x80 | int(get_number_code(counter_GRx10))
            if counter_GHx10 == 0:
                dig_GHx10 = 0x0
            else:
                dig_GHx10 = 0x7F & int(get_number_code(counter_GHx10))
        else:
            dig_GRx1 = 0x80 | int(get_number_code(counter_GRx1))
            if counter_GRx10 == 0:
                dig_GRx10 = 0x80
            else:
                dig_GRx10 = 0x80 | int(get_number_code(counter_GRx10))
            if counter_GHx10 == 0:
                dig_GHx10 = 0x80
            else:
                dig_GHx10 = 0x80 | int(get_number_code(counter_GHx10))
    else:
        if sbvar.display_mode == 1:
            dig_GRx1 = 0x7F & int(get_number_code(counter_GRx1))
        else:
            dig_GRx1 = 0x0
        if counter_GRx10 == 0:
            dig_GRx10 = 0x0
        else:
            dig_GRx10 = 0x7F & int(get_number_code(counter_GRx10))
        if counter_GHx10 == 0:
            dig_GHx10 = 0x0
        else:
            dig_GHx10 = 0x7F & int(get_number_code(counter_GHx10))

    call_string = "i2cset -y" + " " + str(i2c_bus_GHx10) + " " + \
        str(i2c_adr_GHx10) + " " + str(i2c_reg_GHx10) + " " + str(dig_GHx10)
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_GRx1) + " " + \
        str(i2c_adr_GRx1) + " " + str(i2c_reg_GRx1) + " " + str(dig_GRx1)
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_GRx10) + " " + \
        str(i2c_adr_GRx10) + " " + str(i2c_reg_GRx10) + " " + str(dig_GRx10)
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_balls()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_colon
#
# Turns on or off the colon. Since it's implemented as bit 7 of the
# GHx1 register, a boolean operation is done if it's to be turned off.
# otherwise it's simply activated.
#
# Arguments: none
#
# Returns: nothing
def refresh_colon():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_colon()')

    counter_GHx1 = sbvar.counter_GH % 10
    i2c_bus_GHx1, i2c_adr_GHx1, i2c_reg_GHx1 = \
    get_segment_address('GHx1')

    if sbvar.show_COLON == 1:
        dig_GHx1 = 0x80
    else:
        if sbvar.show_GH == 1:
            dig_GHx1 = 0x7F & int(get_number_code(counter_GHx1))
        else:
            dig_GHx1 = 0x0

    call_string = "i2cset -y" + " " + str(i2c_bus_GHx1) + " " + \
        str(i2c_adr_GHx1) + " " + str(i2c_reg_GHx1) + " " + str(dig_GHx1)
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_colon()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_errors
#
# Sends the corresponding i2c commands to display the error counter for
# the team reference given as argument. Also turns the digit on or off,
# depending on the visibility flag. Blinks the updated digits a few
# times, if the flash argument is not 0.
#
# Arguments: 
#   seg = team (GE|HE) [text]
#   flash = (0|1) [integer]
#
# Returns: nothing
def refresh_errors(seg,flash):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_errors(seg,flash), arg:', sbvar.col_fs, seg, flash, sbvar.col_n)

    i2c_bus, i2c_adr, i2c_reg = get_segment_address(seg)

    if seg =='GE':
        if sbvar.show_GE !=0:
            outval = str(get_number_code(sbvar.counter_GE))
        else:
            outval = str(0x0)
    elif seg == 'HE':
        if sbvar.show_HE !=0:
            outval = str(get_number_code(sbvar.counter_HE))
        else:
            outval = str(0x0)
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'refresh_errors(): illegal segment provided:', sbvar.col_er, seg, sbvar.col_n)

    if flash != 0:
        for n in range(5):
            call_string = "i2cset -y" + " " + str(i2c_bus) + " " + \
                str(i2c_adr) + " " + str(i2c_reg) + " " + str(0x0)
            os.system(call_string)
            time.sleep(0.06)
            call_string = "i2cset -y" + " " + str(i2c_bus) + " " + \
                str(i2c_adr) + " " + str(i2c_reg) + " " + outval
            os.system(call_string)
            time.sleep(0.13)
    else:
            call_string = "i2cset -y" + " " + str(i2c_bus) + " " + \
                str(i2c_adr) + " " + str(i2c_reg) + " " + outval
            os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_errors(seg,flash)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_hits
#
# Sends the corresponding i2c commands to display the total hits for
# the team reference given as argument. Also turns the digit on or off,
# depending on the visibility flag. Blinks the updated digits a few
# times, if the "flash" argument is set to 1.
#
# Since the hits digits also accommodate the BALLS 3, OUTS 1 and OUTS 2
# lamps, the state of those counters must also be considered.
#
# Arguments: 
#   seg = team (G|H) [text]
#   flash = (0|1) [integer]
#
# Returns: nothing
def refresh_hits(seg,flash):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, ' refresh_hits(seg,flash), arg:', sbvar.col_fs, seg, flash, sbvar.col_n)

    counter_GHx1 = sbvar.counter_GH % 10
    counter_GHx10 = sbvar.counter_GH // 10
    counter_HHx1 = sbvar.counter_HH % 10
    counter_HHx10 = sbvar.counter_HH // 10

    if seg == 'G':
        i2c_bus_1,  i2c_adr_1,  i2c_reg_1  = get_segment_address('GHx1')
        i2c_bus_10, i2c_adr_10, i2c_reg_10 = get_segment_address('GHx10')
        if sbvar.show_GH !=0:
            # No lamp associated with GHx1 except COLON which is always
            # off when hits are shown
            outval_1 = str(get_number_code(counter_GHx1))
            show_lamp_1 = 0
            if counter_GHx10 == 0:
                if sbvar.counter_BALLS == 3:
                    outval_10 = str(0x80)
                    show_lamp_10 = 1
                else:
                    outval_10 = str(0x0)
                    show_lamp_10 = 0
            else:
                if sbvar.counter_BALLS == 3:
                    outval_10 = str(0x80 | int(get_number_code(counter_GHx10)))
                    show_lamp_10 = 1
                else:
                    outval_10 = str(get_number_code(counter_GHx10))
                    show_lamp_10 = 0
        else:
            outval_1 = str(0x0)
            outval_10 = str(0x0)
            show_lamp_1 = 0
            if sbvar.counter_BALLS == 3:
                show_lamp_10 = 1
            else:
                show_lamp_10 = 0
    elif seg == 'H':
        i2c_bus_1,  i2c_adr_1,  i2c_reg_1  = get_segment_address('HHx1')
        i2c_bus_10, i2c_adr_10, i2c_reg_10 = get_segment_address('HHx10')
        if sbvar.show_HH !=0:
            if sbvar.counter_OUTS == 2:
                outval_1 = str(0x80 | int(get_number_code(counter_HHx1)))
                show_lamp_1 = 1
            else:
                outval_1 = str(get_number_code(counter_HHx1))
                show_lamp_1 = 0
            if counter_HHx10 == 0:
                if sbvar.counter_OUTS > 0:
                    outval_10 = str(0x80)
                    show_lamp_10 = 1
                else:
                    outval_10 = str(0x0)
                    show_lamp_10 = 0
            else:
                if sbvar.counter_OUTS > 0:
                    outval_10 = str(0x80 | int(get_number_code(counter_HHx10)))
                    show_lamp_10 = 1
                else:
                    outval_10 = str(get_number_code(counter_HHx10))
                    show_lamp_10 = 0
        else:
            outval_1 = str(0x0)
            outval_10 = str(0x0)
            if sbvar.counter_OUTS == 2:
                show_lamp_1 = 1
            else:
                show_lamp_1 = 0
            if sbvar.counter_OUTS > 0:
                show_lamp_10 = 1
            else:
                show_lamp_10 = 0
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'refresh_hits(): illegal team indicator provided:', sbvar.col_er, seg, sbvar.col_n)
    
    if flash != 0:
        for n in range(5):
            # Turn off, but leave the BALLS/OUTs lamps on
            if show_lamp_1 == 1:
                call_string = "i2cset -y" + " " + str(i2c_bus_1) + " " + \
                    str(i2c_adr_1) + " " + str(i2c_reg_1) + " " + str(0x80)
            else:
                call_string = "i2cset -y" + " " + str(i2c_bus_1) + " " + \
                    str(i2c_adr_1) + " " + str(i2c_reg_1) + " " + str(0x0)
            os.system(call_string)

            if show_lamp_10 == 1:
                call_string = "i2cset -y" + " " + str(i2c_bus_10) + " " + \
                    str(i2c_adr_10) + " " + str(i2c_reg_10) + " " + str(0x80)
            else:
                call_string = "i2cset -y" + " " + str(i2c_bus_10) + " " + \
                    str(i2c_adr_10) + " " + str(i2c_reg_10) + " " + str(0x0)
            os.system(call_string)
            time.sleep(0.03)
            
            # Turn on
            call_string = "i2cset -y" + " " + str(i2c_bus_1) + " " + \
                str(i2c_adr_1) + " " + str(i2c_reg_1) + " " + outval_1
            os.system(call_string)

            call_string = "i2cset -y" + " " + str(i2c_bus_10) + " " + \
                str(i2c_adr_10) + " " + str(i2c_reg_10) + " " + outval_10
            os.system(call_string)
            time.sleep(0.08)
    else:
            call_string = "i2cset -y" + " " + str(i2c_bus_1) + " " + \
                str(i2c_adr_1) + " " + str(i2c_reg_1) + " " + outval_1
            os.system(call_string)
            call_string = "i2cset -y" + " " + str(i2c_bus_10) + " " + \
                str(i2c_adr_10) + " " + str(i2c_reg_10) + " " + outval_10
            os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_hits(seg,flash)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_inning_runs
#
# Sends the corresponding i2c commands to display the inning run counter 
# for the half-inning digit given as an argument. Also turns the digit 
# on or off, depending on the visibility flag.
#
# In Baseball and Tossball mode, if the current inning is 7 or lower,
# displays an "A" and a "H" in G9/H9. For Top innings, displays a "-"
# in G8 and GX, and in H8 and HX for Bottom innings.
#
# Arguments: 
#   seg = half-inning designator [text]
#
# Returns: nothing
def refresh_inning_runs(seg):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_inning_runs(seg), arg:', sbvar.col_fs, seg, sbvar.col_n)

    # Get the i2c addressing values:
    i2c_bus, i2c_adr, i2c_reg = get_segment_address(seg)

    # In the if structure, the variable "outval" is filled with the hex
    # value corresponding to the current counter value
    if seg =='G1':
        if sbvar.show_G1 != 0:
            # If the variable to hide or show the digit is not 0 = 
            # show the digit
            outval = str(get_number_code(sbvar.counter_G1))
        else:
            # If the variable is 0 = hide the digit by sending 0x0
            outval = str(0x0)
    elif seg == 'G2':
        if sbvar.show_G2 != 0:
            outval = str(get_number_code(sbvar.counter_G2))
        else:
            outval = str(0x0)
    elif seg == 'G3':
        if sbvar.show_G3 != 0:
            outval = str(get_number_code(sbvar.counter_G3))
        else:
            outval = str(0x0)
    elif seg == 'G4':
        if sbvar.show_G4 != 0:
            outval = str(get_number_code(sbvar.counter_G4))
        else:
            outval = str(0x0)
    elif seg == 'G5':
        if sbvar.show_G5 != 0:
            outval = str(get_number_code(sbvar.counter_G5))
        else:
            outval = str(0x0)
    elif seg == 'G6':
        if sbvar.show_G6 != 0:
            outval = str(get_number_code(sbvar.counter_G6))
        else:
            outval = str(0x0)
    elif seg == 'G7':
        if sbvar.show_G7 != 0:
            outval = str(get_number_code(sbvar.counter_G7))
        else:
            outval = str(0x0)
    elif seg == 'G8':
        if sbvar.show_G8 != 0:
            outval = str(get_number_code(sbvar.counter_G8))
        elif sbvar.display_mode != 0 and sbvar.counter_INNING in [1,3,5,7,9,11]:
            outval = str(0x40)
        else:
            outval = str(0x0)
    elif seg == 'G9':
        if sbvar.show_G9 != 0:
            outval = str(get_number_code(sbvar.counter_G9))
        elif sbvar.counter_INNING <= 14 and sbvar.display_mode != 0:
            outval = str(0x77)
        else:
            outval = str(0x0)
    elif seg == 'GX':
        if sbvar.show_GX != 0:
            outval = str(get_number_code(sbvar.counter_GX))
        elif sbvar.display_mode != 0 and sbvar.counter_INNING in [1,3,5,7,9,11,13,15]:
            outval = str(0x40)
        else:
            outval = str(0x0)
    elif seg == 'H1':
        if sbvar.show_H1 != 0:
            outval = str(get_number_code(sbvar.counter_H1))
        else:
            outval = str(0x0)
    elif seg == 'H2':
        if sbvar.show_H2 != 0:
            outval = str(get_number_code(sbvar.counter_H2))
        else:
            outval = str(0x0)
    elif seg == 'H3':
        if sbvar.show_H3 != 0:
            outval = str(get_number_code(sbvar.counter_H3))
        else:
            outval = str(0x0)
    elif seg == 'H4':
        if sbvar.show_H4 != 0:
            outval = str(get_number_code(sbvar.counter_H4))
        else:
            outval = str(0x0)
    elif seg == 'H5':
        if sbvar.show_H5 != 0:
            outval = str(get_number_code(sbvar.counter_H5))
        else:
            outval = str(0x0)
    elif seg == 'H6':
        if sbvar.show_H6 != 0:
            outval = str(get_number_code(sbvar.counter_H6))
        else:
            outval = str(0x0)
    elif seg == 'H7':
        if sbvar.show_H7 != 0:
            outval = str(get_number_code(sbvar.counter_H7))
        else:
            outval = str(0x0)
    elif seg == 'H8':
        if sbvar.show_H8 != 0:
            outval = str(get_number_code(sbvar.counter_H8))
        elif sbvar.display_mode != 0 and sbvar.counter_INNING in [2,4,6,8,10,12]:
            outval = str(0x40)
        else:
            outval = str(0x0)
    elif seg == 'H9':
        if sbvar.show_H9 != 0:
            outval = str(get_number_code(sbvar.counter_H9))
        elif sbvar.counter_INNING <= 14 and sbvar.display_mode != 0:
            outval = str(0x76)
        else:
            outval = str(0x0)
    elif seg == 'HX':
        if sbvar.show_HX != 0:
            outval = str(get_number_code(sbvar.counter_HX))
        elif sbvar.display_mode != 0 and sbvar.counter_INNING in [2,4,6,8,10,12,14,16]:
            outval = str(0x40)
        else:
            outval = str(0x0)
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'refresh_inning_runs(): illegal digit provided:', sbvar.col_er, seg, sbvar.col_n)

    # Send the value to the i2c driver address
    call_string = "i2cset -y" + " " + str(i2c_bus) + " " + \
        str(i2c_adr) + " " + str(i2c_reg) + " " + outval
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_inning_runs(seg)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_outs
#
# Sends the corresponding i2c commands to turn the two out lamps on or 
# off depending on the value of the sbvar.counter_OUTS counter. Since 
# the lamps are implemented as bit 7 of the HHx1 and HHx10 registers, a
# boolean operation is first done to not disrupt the HHx1 or HHx10 
# digits.
#
# Arguments: none
#
# Returns: nothing
def refresh_outs():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_outs()')

    # Get the 1x-digit of the counter for Home Hits
    counter_HHx1 = sbvar.counter_HH % 10
    # Get the 10x-digit of the counter for Home Hits
    counter_HHx10 = sbvar.counter_HH // 10
    # Get the i2c addressing values for the two segment drivers
    i2c_bus_HHx1, i2c_adr_HHx1, i2c_reg_HHx1 = get_segment_address('HHx1')
    i2c_bus_HHx10, i2c_adr_HHx10, i2c_reg_HHx10 = get_segment_address('HHx10')

    # Show indicators depending on the display mode
    if sbvar.display_mode == 0:
    # Time mode (0) = don't show outs or hits
        dig_HHx10 = 0x0
        dig_HHx1 = 0x0
    elif sbvar.display_mode == 1:
    # Baseball mode (1) = show both outs and hits
        if sbvar.counter_OUTS == 0:
            # Do a boolean AND with '01111111' to force the 7th bit to 0
            # for the OUTS 2 lamp
            dig_HHx1 = 0x7F & int(get_number_code(counter_HHx1))
            # For the OUTS 1 lamp, do the same for the HHx10 counter
            # Also turn off the HHx10 digit if there are less than 10 hits
            if counter_HHx10 == 0:
                dig_HHx10 = 0x0
            else:
                dig_HHx10 = 0x7F & int(get_number_code(counter_HHx10))
        # One out = turn on OUTS 1, leave OUTS 2 off
        # This is done with a boolean OR with '10000000'
        elif sbvar.counter_OUTS == 1:
            dig_HHx1 = 0x7F & int(get_number_code(counter_HHx1))
            if counter_HHx10 == 0:
                dig_HHx10 = 0x80
            else:
                dig_HHx10 = 0x80 | int(get_number_code(counter_HHx10))
        else:
            dig_HHx1 = 0x80 | int(get_number_code(counter_HHx1))
            if counter_HHx10 == 0:
                dig_HHx10 = 0x80
            else:
                dig_HHx10 = 0x80 | int(get_number_code(counter_HHx10))
    else:
    # Teeball mode (2): show only outs, not hits
        if sbvar.counter_OUTS == 0:
            dig_HHx1 = 0x0
            dig_HHx10 = 0x0
        elif sbvar.counter_OUTS == 1:
            dig_HHx1 = 0x0
            dig_HHx10 = 0x80
        else:
            dig_HHx1 = 0x80
            dig_HHx10 = 0x80

    # Send the calculated digit values to the i2c bus drivers:
    # Assemble the string to send
    call_string = "i2cset -y" + " " + str(i2c_bus_HHx1) + " " + \
        str(i2c_adr_HHx1) + " " + str(i2c_reg_HHx1) + " " + str(dig_HHx1)
    # Execute the system call
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_HHx10) + " " + \
        str(i2c_adr_HHx10) + " " + str(i2c_reg_HHx10) + " " + str(dig_HHx10)
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_outs()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_runs
#
# Sends the corresponding i2c commands to display the total runs for
# the team reference given as argument. Also turns the digit on or off,
# depending on the visibility flag. 
#
# Since the runs digits also accommodate the BALLS 1, BALLS 2, STRIKES 1
# and STRIKES 2 lamps, the state of those counters must also be 
# considered.
#
# Arguments: 
#  seg = team (G|H) [text]
#
# Returns: nothing
def refresh_runs(seg):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_runs(seg), arg:', sbvar.col_fs, seg, sbvar.col_n)

    counter_GRx1 = sbvar.counter_GR % 10
    counter_GRx10 = sbvar.counter_GR // 10
    counter_HRx1 = sbvar.counter_HR % 10
    counter_HRx10 = sbvar.counter_HR // 10

    if seg == 'G':
        i2c_bus_1, i2c_adr_1, i2c_reg_1 = get_segment_address('GRx1')
        i2c_bus_10, i2c_adr_10, i2c_reg_10 = get_segment_address('GRx10')
        if sbvar.show_GR !=0:
            if sbvar.counter_BALLS > 1:
                outval_1 = str(0x80 | int(get_number_code(counter_GRx1)))
            else:
                outval_1 = str(get_number_code(counter_GRx1))
            if counter_GRx10 == 0:
                if sbvar.counter_BALLS > 0:
                    outval_10 = str(0x80)
                else:
                    outval_10 = str(0x0)
            else:
                if sbvar.counter_BALLS > 0:
                    outval_10 = str(0x80 | int(get_number_code(counter_GRx10)))
                else:
                    outval_10 = str(get_number_code(counter_GRx10))
        else:
            outval_1 = str(0x0)
            outval_10 = str(0x0)
    elif seg == 'H':
        i2c_bus_1, i2c_adr_1, i2c_reg_1 = get_segment_address('HRx1')
        i2c_bus_10, i2c_adr_10, i2c_reg_10 = get_segment_address('HRx10')
        if sbvar.show_HR !=0:
            if sbvar.counter_STRIKES == 2:
                outval_1 = str(0x80 | int(get_number_code(counter_HRx1)))
            else:
                outval_1 = str(get_number_code(counter_HRx1))
            if counter_HRx10 == 0:
                if sbvar.counter_STRIKES > 0:
                    outval_10 = str(0x80)
                else:
                    outval_10 = str(0x0)
            else:
                if sbvar.counter_STRIKES > 0:
                    outval_10 = str(0x80 | int(get_number_code(counter_HRx10)))
                else:
                    outval_10 = str(get_number_code(counter_HRx10))
        else:
            outval_1 = str(0x0)
            outval_10 = str(0x0)
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'refresh_runs(seg): illegal team indicator provided:', sbvar.col_er, seg, sbvar.col_n)
    
    call_string = "i2cset -y" + " " + str(i2c_bus_1) + " " + \
        str(i2c_adr_1) + " " + str(i2c_reg_1) + " " + outval_1
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_10) + " " + \
        str(i2c_adr_10) + " " + str(i2c_reg_10) + " " + outval_10
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_runs(seg)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# refresh_strikes
#
# Sends the corresponding i2c commands to turn the two strikes lamps on 
# or off depending on the value of the sbvar.counter_STRIKES counter. 
# Since the lamps are implemented as bit 7 of the HRx1 and HRx10 
# registers, a boolean operation is first done to not disrupt the HRx1 
# or HRx10 digits.
#
# See comments in the refresh_outs() function for details.
#
# Arguments: none
#
# Returns: nothing
def refresh_strikes():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'refresh_strikes()')

    counter_HRx1 = sbvar.counter_HR % 10
    counter_HRx10 = sbvar.counter_HR // 10
    i2c_bus_HRx1, i2c_adr_HRx1, i2c_reg_HRx1 = \
        get_segment_address('HRx1')
    i2c_bus_HRx10, i2c_adr_HRx10, i2c_reg_HRx10 = \
        get_segment_address('HRx10')

    if sbvar.show_STRIKES != 0:
        if sbvar.counter_STRIKES == 0:
            dig_HRx1 = 0x7F & int(get_number_code(counter_HRx1))
            if counter_HRx10 == 0:
                dig_HRx10 = 0x0
            else:
                dig_HRx10 = 0x7F & int(get_number_code(counter_HRx10))
        elif sbvar.counter_STRIKES == 1:
            dig_HRx1 = 0x7F & int(get_number_code(counter_HRx1))
            if counter_HRx10 == 0:
                dig_HRx10 = 0x80
            else:
                dig_HRx10 = 0x80 | int(get_number_code(counter_HRx10))
        else:
            dig_HRx1 = 0x80 | int(get_number_code(counter_HRx1))
            if counter_HRx10 == 0:
                dig_HRx10 = 0x80
            else:
                dig_HRx10 = 0x80 | int(get_number_code(counter_HRx10))
    else:
        if sbvar.display_mode == 1:
            dig_HRx1 = 0x7F & int(get_number_code(counter_HRx1))
        else:
            dig_HRx1 = 0x0
        if counter_HRx10 == 0:
            dig_HRx10 = 0x0
        else:
            dig_HRx10 = 0x7F & int(get_number_code(counter_HRx10))

    call_string = "i2cset -y" + " " + str(i2c_bus_HRx1) + " " + \
        str(i2c_adr_HRx1) + " " + str(i2c_reg_HRx1) + " " + str(dig_HRx1)
    os.system(call_string)
    call_string = "i2cset -y" + " " + str(i2c_bus_HRx10) + " " + \
        str(i2c_adr_HRx10) + " " + str(i2c_reg_HRx10) + " " + str(dig_HRx10)
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'refresh_strikes()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_balls_count
#
# This function updates the BALLS counter by  incrementing by one, and 
# refreshes the corresponding digit. The counter snaps to 0 if incremented 
# from 3. It also refreshes the display.
#
# Arguments: None
#
# Returns: nothing
#
def update_balls_count():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_balls_count()')

    if sbvar.display_mode == 1:
        if sbvar.counter_INNING == 0:
            if sbvar.debug_mode != 0:
                exec_time = (datetime.datetime.now() - start_time)
                print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_balls_count() - no update if in Pre-Game', sbvar.col_t, exec_time, sbvar.col_n)
            return None

        if sbvar.counter_BALLS == 3:
            sbvar.counter_BALLS = 0
        else:
            sbvar.counter_BALLS += 1
        refresh_balls()
    else:
        print(sbvar.col_wa, '    WARNING', sbvar.col_n, 'update_balls_count(): no update if not in Baseball mode')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_balls_count()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_error_count
#
# This function updates an error counter by either incrementing or
# decrementing by one, and refreshes the corresponding digit.
#
# Arguments:
#   team (GE|HE) [string]
#   direction (up|down) [string]
#   flash [boolean]
#
# Returns: nothing
#
def update_error_count(team,direction,flash):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_error_count(team,direction,flash), arg:', sbvar.col_fs, team, direction, flash, sbvar.col_n)

    if sbvar.display_mode == 1:
        if sbvar.counter_INNING == 0:
            if sbvar.debug_mode != 0:
                exec_time = (datetime.datetime.now() - start_time)
                print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_error_count(team,direction,flash) - no update if in Pre-Game', sbvar.col_t, exec_time, sbvar.col_n)
            return None

        if direction != 'up' and direction != 'down':
            print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_error_count(): illegal direction given:', sbvar.col_er, direction, sbvar.col_n)
            return None
        if team == 'GE':
            if direction == 'down':
                flash = 0
                if sbvar.counter_GE > 0:
                    sbvar.counter_GE -= 1
            else:
                sbvar.counter_GE += 1
            refresh_errors('GE',flash)
        elif team == 'HE':
            if direction == 'down':
                flash = 0
                if sbvar.counter_HE > 0:
                    sbvar.counter_HE -= 1
            else:
                sbvar.counter_HE += 1
            refresh_errors('HE',flash)
        else:
            print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_error_count(): illegal team provided - must be "GE" or "HE":', sbvar.col_er, team, sbvar.col_n)
    else:
        print(sbvar.col_wa, '    WARNING:', sbvar.col_n, 'update_error_count() - no update if not in Baseball mode')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_error_count(team,direction,flash)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_given_run_counter
#
# This function updates one inning run counter by either incrementing or 
# decrementing by one, and refreshes the corresponding digit.
#
# Arguments:
#    inning = halfinning label [string]
#    direction = count up or down (up|down) [string]
#
# Returns: nothing
#
def update_given_run_counter(inning,direction):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_given_run_counter(inning,direction), arg:', sbvar.col_fs, inning, direction, sbvar.col_n)

    if direction != 'up' and direction != 'down':
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_given_run_counter(): Illegal direction given:', sbvar.col_er, direction, sbvar.col_n)
        return None

    if inning == 'G1':
        update_inning_runs(1,direction)
    elif inning == 'H1':
        update_inning_runs(2,direction)
    elif inning == 'G2':
        update_inning_runs(3,direction)
    elif inning == 'H2':
        update_inning_runs(4,direction)
    elif inning == 'G3':
        update_inning_runs(5,direction)
    elif inning == 'H3':
        update_inning_runs(6,direction)
    elif inning == 'G4':
        update_inning_runs(7,direction)
    elif inning == 'H4':
        update_inning_runs(8,direction)
    elif inning == 'G5':
        update_inning_runs(9,direction)
    elif inning == 'H5':
        update_inning_runs(10,direction)
    elif inning == 'G6':
        update_inning_runs(11,direction)
    elif inning == 'H6':
        update_inning_runs(12,direction)
    elif inning == 'G7':
        update_inning_runs(13,direction)
    elif inning == 'H7':
        update_inning_runs(14,direction)
    elif inning == 'G8':
        update_inning_runs(15,direction)
    elif inning == 'H8':
        update_inning_runs(16,direction)
    elif inning == 'G9':
        update_inning_runs(17,direction)
    elif inning == 'H9':
        update_inning_runs(18,direction)
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_given_run_counter(): Invalid inning label given:', sbvar.col_er, inning, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_given_run_counter(inning,direction)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_hits_count
#
# This function updates an hits counter by either incrementing or
# decrementing by one, and refreshes the corresponding digit.
#
# Arguments:
#   team (G|H) [string]
#   direction (up|down) [string]
#   flash [boolean]
#
# Returns: nothing
#
def update_hits_count(team,direction,flash):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_hits_count(team,direction,flash), arg:', sbvar.col_fs, team, direction, flash, sbvar.col_n)

    if sbvar.display_mode == 1:
        if sbvar.counter_INNING == 0:
            if sbvar.debug_mode != 0:
                exec_time = (datetime.datetime.now() - start_time)
                print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_hits_count(team,direction,flash) - no update if in Pre-Game', sbvar.col_t, exec_time, sbvar.col_n)
            return None

        if direction != 'up' and direction != 'down':
            print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_error_count(): illegal direction given:', sbvar.col_er, direction, sbvar.col_n)
            return None
        if team == 'G':
            if direction == 'down':
                flash = 0
                if sbvar.counter_GH > 0:
                    sbvar.counter_GH -= 1
            else:
                sbvar.counter_GH += 1
            refresh_hits('G',flash)
        elif team == 'H':
            if direction == 'down':
                flash = 0
                if sbvar.counter_HH > 0:
                    sbvar.counter_HH -= 1
            else:
                sbvar.counter_HH += 1
            refresh_hits('H',flash)
        else:
            print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_hits_count(): illegal team provided - must be "G" or "H":', sbvar.col_er, team, sbvar.col_n)
    else:
        print(sbvar.col_wa, '    WARNING', sbvar.col_n, 'update_hits_count() - no update if not in Baseball mode')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_hits_count(team,direction,flash)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_inning_count
#
# Increments or decrements the counter indicating the current inning.
# Also updates the show_XX variables to show or hide the inning run
# digits. Refreshes the inning run digits.
#
# Arguments:
#   direction  (up|down|refresh) [string]
#
# Returns: nothing
#
def update_inning_count(direction):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_inning_count(direction), arg:', sbvar.col_fs, direction, sbvar.col_n)

    if direction != 'up' and direction != 'down' and direction != 'refresh':
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_inning_count(): illegal direction given:', sbvar.col_er, direction, sbvar.col_n)
        return None
    
    # Don't do anything if time mode is active
    if sbvar.display_mode == 0:
        if sbvar.debug_mode != 0:
            exec_time = (datetime.datetime.now() - start_time)
            print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_inning_count(direction) - no update in time mode.', sbvar.col_t, exec_time, sbvar.col_n)
        return None
    
    all_innings = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                   'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX']
    
    # Increment or decrement the inning counter
    if direction == 'up':
        sbvar.counter_INNING += 1
    elif direction == 'down':
        if sbvar.counter_INNING != 0:
            sbvar.counter_INNING -= 1
    else:
        pass
    
    # Turn on or off the inning digits
    if sbvar.counter_INNING == 0:
        for i in all_innings:
            hide_inning(i)
    elif sbvar.counter_INNING == 1:
        inning_list = ['G1']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 2:
        inning_list = ['G1','H1']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 3:
        inning_list = ['G1','H1','G2']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 4:
        inning_list = ['G1','H1','G2','H2']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 5:
        inning_list = ['G1','H1','G2','H2','G3']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 6:
        inning_list = ['G1','H1','G2','H2','G3','H3']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 7:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 8:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 9:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 10:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 11:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 12:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 13:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 14:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 15:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7','G8']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 16:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7','G8','H8']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 17:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7','G8','H8','G9']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 18:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7','G8','H8','G9','H9']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    elif sbvar.counter_INNING == 19:
        inning_list = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                       'G6','H6','G7','H7','G8','H8','G9','H9','GX']
        for i in all_innings:
            hide_inning(i)
        for i in inning_list:
            show_inning(i)
    else:
        for i in all_innings:
            show_inning(i)
  
    refresh_inning_runs('G1')
    refresh_inning_runs('H1')
    refresh_inning_runs('G2')
    refresh_inning_runs('H2')
    refresh_inning_runs('G3')
    refresh_inning_runs('H3')
    refresh_inning_runs('G4')
    refresh_inning_runs('H4')
    refresh_inning_runs('G5')
    refresh_inning_runs('H5')
    refresh_inning_runs('G6')
    refresh_inning_runs('H6')
    refresh_inning_runs('G7')
    refresh_inning_runs('H7')
    refresh_inning_runs('G8')
    refresh_inning_runs('H8')
    refresh_inning_runs('G9')
    refresh_inning_runs('H9')
    refresh_inning_runs('GX')
    refresh_inning_runs('HX')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_inning_count(direction)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_inning_runs
#
# This function updates one inning run counter by either incrementing or 
# decrementing by one, and refreshes the corresponding digit. It also
# triggers an update of the corresponding run counter.
#
# Arguments:
#    inning = halfinning number [integer]
#    direction = count up or down (up|down) [string]
#
# Returns: nothing
#
def update_inning_runs(inning,direction):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_inning_runs(inning,direction), arg:', sbvar.col_fs, inning, direction, sbvar.col_n)

    if direction != 'up' and direction != 'down':
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_inning_runs(): illegal direction given:', sbvar.col_er, direction, sbvar.col_n)
        return None
    if inning == 0:
        print(sbvar.col_wa, 'WARNING', sbvar.col_n, 'update_inning_runs(): no inning run updated in Pre-Game mode')
    elif inning == 1:
        if direction == 'down':
            if sbvar.counter_G1 > 0:
                sbvar.counter_G1 -= 1
        else:
            sbvar.counter_G1 += 1
        refresh_inning_runs('G1')
        update_run_counter('G')
    elif inning == 2:
        if direction == 'down':
            if sbvar.counter_H1 > 0:
                sbvar.counter_H1 -= 1
        else:
            sbvar.counter_H1 += 1
        refresh_inning_runs('H1')
        update_run_counter('H')
    elif inning == 3:
        if direction == 'down':
            if sbvar.counter_G2 > 0:
                sbvar.counter_G2 -= 1
        else:
            sbvar.counter_G2 += 1
        refresh_inning_runs('G2')
        update_run_counter('G')
    elif inning == 4:
        if direction == 'down':
            if sbvar.counter_H2 > 0:
                sbvar.counter_H2 -= 1
        else:
            sbvar.counter_H2 += 1
        refresh_inning_runs('H2')
        update_run_counter('H')
    elif inning == 5:
        if direction == 'down':
            if sbvar.counter_G3 > 0:
                sbvar.counter_G3 -= 1
        else:
            sbvar.counter_G3 += 1
        refresh_inning_runs('G3')
        update_run_counter('G')
    elif inning == 6:
        if direction == 'down':
            if sbvar.counter_H3 > 0:
                sbvar.counter_H3 -= 1
        else:
            sbvar.counter_H3 += 1
        refresh_inning_runs('H3')
        update_run_counter('H')
    elif inning == 7:
        if direction == 'down':
            if sbvar.counter_G4 > 0:
                sbvar.counter_G4 -= 1
        else:
            sbvar.counter_G4 += 1
        refresh_inning_runs('G4')
        update_run_counter('G')
    elif inning == 8:
        if direction == 'down':
            if sbvar.counter_H4 > 0:
                sbvar.counter_H4 -= 1
        else:
            sbvar.counter_H4 += 1
        refresh_inning_runs('H4')
        update_run_counter('H')
    elif inning == 9:
        if direction == 'down':
            if sbvar.counter_G5 > 0:
                sbvar.counter_G5 -= 1
        else:
            sbvar.counter_G5 += 1
        refresh_inning_runs('G5')
        update_run_counter('G')
    elif inning == 10:
        if direction == 'down':
            if sbvar.counter_H5 > 0:
                sbvar.counter_H5 -= 1
        else:
            sbvar.counter_H5 += 1
        refresh_inning_runs('H5')
        update_run_counter('H')
    elif inning == 11:
        if direction == 'down':
            if sbvar.counter_G6 > 0:
                sbvar.counter_G6 -= 1
        else:
            sbvar.counter_G6 += 1
        refresh_inning_runs('G6')
        update_run_counter('G')
    elif inning == 12:
        if direction == 'down':
            if sbvar.counter_H6 > 0:
                sbvar.counter_H6 -= 1
        else:
            sbvar.counter_H6 += 1
        refresh_inning_runs('H6')
        update_run_counter('H')
    elif inning == 13:
        if direction == 'down':
            if sbvar.counter_G7 > 0:
                sbvar.counter_G7 -= 1
        else:
            sbvar.counter_G7 += 1
        refresh_inning_runs('G7')
        update_run_counter('G')
    elif inning == 14:
        if direction == 'down':
            if sbvar.counter_H7 > 0:
                sbvar.counter_H7 -= 1
        else:
            sbvar.counter_H7 += 1
        refresh_inning_runs('H7')
        update_run_counter('H')
    elif inning == 15:
        if direction == 'down':
            if sbvar.counter_G8 > 0:
                sbvar.counter_G8 -= 1
        else:
            sbvar.counter_G8 += 1
        refresh_inning_runs('G8')
        update_run_counter('G')
    elif inning == 16:
        if direction == 'down':
            if sbvar.counter_H8 > 0:
                sbvar.counter_H8 -= 1
        else:
            sbvar.counter_H8 += 1
        refresh_inning_runs('H8')
        update_run_counter('H')
    elif inning == 17:
        if direction == 'down':
            if sbvar.counter_G9 > 0:
                sbvar.counter_G9 -= 1
        else:
            sbvar.counter_G9 += 1
        refresh_inning_runs('G9')
        update_run_counter('G')
    elif inning == 18:
        if direction == 'down':
            if sbvar.counter_H9 > 0:
                sbvar.counter_H9 -= 1
        else:
            sbvar.counter_H9 += 1
        refresh_inning_runs('H9')
        update_run_counter('H')
    # For all extra innings, use either the one for GUEST or HOME 
    # alternatively
    elif inning > 18 and inning % 2 != 0:
        if direction == 'down':
            if sbvar.counter_GX > 0:
                sbvar.counter_GX -= 1
        else:
            sbvar.counter_GX += 1
        refresh_inning_runs('GX')
        update_run_counter('G')
    elif inning > 18 and inning % 2 == 0:
        if direction == 'down':
            if sbvar.counter_HX > 0:
                sbvar.counter_HX -= 1
        else:
            sbvar.counter_HX += 1
        refresh_inning_runs('HX')
        update_run_counter('H')
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, '(update_inning_runs): Invalid inning number given:', sbvar.col_er, inning, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_inning_runs(inning,direction)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_outs_count
#
# This function updates the OUTS counter by  incrementing by one, and 
# refreshes the corresponding digit. The counter snaps to 0 if incremented 
# from 2. It also refreshes the display.
#
# Arguments: None
#
# Returns: nothing
#
def update_outs_count():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_outs_count()')

    if sbvar.counter_INNING == 0:
        if sbvar.debug_mode != 0:
            exec_time = (datetime.datetime.now() - start_time)
            print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_outs_count() - no update if in Pre-Game', sbvar.col_t, exec_time, sbvar.col_n)
        return None
    
    if sbvar.counter_OUTS == 2:
        sbvar.counter_OUTS = 0
    else:
        sbvar.counter_OUTS += 1
    refresh_outs()

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_outs_count()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_run_counter
#
# This function sets the Runs counter for the team given in the input
# argument to be the sum of all that team's inning counters.
#
# Arguments: 
#   team = (H|G) [string]
#
# Returns: nothing
#
def update_run_counter(team):
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_run_counter(team), arg:', sbvar.col_fs, team, sbvar.col_n)

    if team == 'G':
        sbvar.counter_GR = sbvar.counter_G1 + \
                     sbvar.counter_G2 + \
                     sbvar.counter_G3 + \
                     sbvar.counter_G4 + \
                     sbvar.counter_G5 + \
                     sbvar.counter_G6 + \
                     sbvar.counter_G7 + \
                     sbvar.counter_G8 + \
                     sbvar.counter_G9 + \
                     sbvar.counter_GX
        refresh_runs('G')
    elif team == 'H':
        sbvar.counter_HR = sbvar.counter_H1 + \
                     sbvar.counter_H2 + \
                     sbvar.counter_H3 + \
                     sbvar.counter_H4 + \
                     sbvar.counter_H5 + \
                     sbvar.counter_H6 + \
                     sbvar.counter_H7 + \
                     sbvar.counter_H8 + \
                     sbvar.counter_H9 + \
                     sbvar.counter_HX
        refresh_runs('H')
    else:
        print(sbvar.col_er, 'ERROR', sbvar.col_n, 'update_run_counter(): wrong team given - must be "H" or "G":', sbvar.col_er, team, sbvar.col_n)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_run_counter(team)', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# update_strikes_count
#
# This function updates the STRIKES counter by  incrementing by one, and 
# refreshes the corresponding digit. The counter snaps to 0 if incremented 
# from 2. It also refreshes the display.
#
# Arguments: None
#
# Returns: nothing
#
def update_strikes_count():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'update_strikes_count()')

    if sbvar.display_mode == 1:
        if sbvar.counter_INNING == 0:
            if sbvar.debug_mode != 0:
                exec_time = (datetime.datetime.now() - start_time)
                print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_strikes_count() - no update if in Pre-Game', sbvar.col_t, exec_time, sbvar.col_n)
            return None

        if sbvar.counter_STRIKES == 2:
            sbvar.counter_STRIKES = 0
        else:
            sbvar.counter_STRIKES += 1
        refresh_strikes()
    else:
        print(sbvar.col_wa, '    WARNING', sbvar.col_n, 'update_strikes_count() - no update if not in Baseball mode')

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'update_strikes_count()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# restart_scoreboard
#
# Restarts the raspberry pi. 
#
# Arguments: none
#
# Returns: nothing
def restart_scoreboard():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'restart_scoreboard()')

    # Display "reboot" message
    # Send 0x0 to all segments first
    all_segs = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
               'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX',\
               'GRx10','GRx1','GHx10','GHx1','GE',\
               'HRx10','HRx1','HHx10','HHx1','HE']
    for i in all_segs:
        show_one_character(i, 0x0) # " "
    
    show_one_character('G5', 0x50) # G5: "r"
    show_one_character('G6', 0x79) # G6: "E"
    show_one_character('G7', 0x7C) # G7: "b"
    show_one_character('G8', 0x5C) # G8: "o"
    show_one_character('G9', 0x5C) # G9: "o"
    show_one_character('GX', 0x78) # GX: "t"

    call_string = 'sudo reboot'
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'restart_scoreboard()', sbvar.col_t, exec_time, sbvar.col_n)
  
    return None



########################################################################
# start_new_game
#
# This function sets all the global counters to 0 and initiates the
# display.
#
# Arguments: none
#
# Returns: nothing
#
def start_new_game():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'start_new_game()')

    # Guest inning runs counters:
    sbvar.counter_G1 = 0
    sbvar.counter_G2 = 0
    sbvar.counter_G3 = 0
    sbvar.counter_G4 = 0
    sbvar.counter_G5 = 0
    sbvar.counter_G6 = 0
    sbvar.counter_G7 = 0
    sbvar.counter_G8 = 0
    sbvar.counter_G9 = 0
    sbvar.counter_GX = 0
    # Home inning runs counters:
    sbvar.counter_H1 = 0
    sbvar.counter_H2 = 0
    sbvar.counter_H3 = 0
    sbvar.counter_H4 = 0
    sbvar.counter_H5 = 0
    sbvar.counter_H6 = 0
    sbvar.counter_H7 = 0
    sbvar.counter_H8 = 0
    sbvar.counter_H9 = 0
    sbvar.counter_HX = 0
    # Error counters:
    sbvar.counter_GE = 0
    sbvar.counter_HE = 0
    # Runs counters:
    sbvar.counter_GR = 0
    sbvar.counter_HR = 0
    # Hits counters:
    sbvar.counter_GH = 0
    sbvar.counter_HH = 0
    # Outs, balls, strikes counters:
    sbvar.counter_OUTS = 0
    sbvar.counter_STRIKES = 0
    sbvar.counter_BALLS = 0
    # Current inning:
    sbvar.counter_INNING = 0

    # Turn off the inning run display:
    all_innings = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
                   'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX']
    for i in all_innings:
        hide_inning(i)
    
    if sbvar.display_mode == 1 or sbvar.display_mode == 2:
        refresh_inning_runs('G1')
        refresh_inning_runs('H1')
        refresh_inning_runs('G2')
        refresh_inning_runs('H2')
        refresh_inning_runs('G3')
        refresh_inning_runs('H3')
        refresh_inning_runs('G4')
        refresh_inning_runs('H4')
        refresh_inning_runs('G5')
        refresh_inning_runs('H5')
        refresh_inning_runs('G6')
        refresh_inning_runs('H6')
        refresh_inning_runs('G7')
        refresh_inning_runs('H7')
        refresh_inning_runs('G8')
        refresh_inning_runs('H8')
        refresh_inning_runs('G9')
        refresh_inning_runs('H9')
        refresh_inning_runs('GX')
        refresh_inning_runs('HX')
        refresh_runs('H')
        refresh_runs('G')
        refresh_outs()

    if sbvar.display_mode == 1:
        refresh_hits('H',0)
        refresh_hits('G',0)
        refresh_errors('HE',0)
        refresh_errors('GE',0)
        refresh_balls()
        refresh_strikes()

    if sbvar.display_mode == 2:
        set_display_mode(2)

    if sbvar.display_mode == 0:
        refresh_colon()

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'start_new_game()', sbvar.col_t, exec_time, sbvar.col_n)

    return None



########################################################################
# shutdown_scoreboard
#
# Shuts down the raspberry pi. The only way to restart the system is
# to cycle the power supply after shutdown.
#
# Arguments: none
#
# Returns: nothing
def shutdown_scoreboard():
    if sbvar.debug_mode != 0:
        start_time = datetime.datetime.now()
        print(sbvar.col_fs, '  FUNC_START:', sbvar.col_n, 'shutdown_scoreboard()')
    
    # Display "turn OFF" message

    # Send 0x0 to all segments first
    all_segs = ['G1','H1','G2','H2','G3','H3','G4','H4','G5','H5',\
               'G6','H6','G7','H7','G8','H8','G9','H9','GX','HX',\
               'GRx10','GRx1','GHx10','GHx1','GE',\
               'HRx10','HRx1','HHx10','HHx1','HE']
    for i in all_segs:
        show_one_character(i, 0x0) # " "
    
    show_one_character('G1', 0xD1) # G1: Arrow turn
    show_one_character('G2', 0x41) # G2: horizontal double line
    show_one_character('G3', 0x41) # G3: horizontal double line
    show_one_character('G4', 0x41) # G4: horizontal double line
    show_one_character('G5', 0x41) # G5: horizontal double line

    show_one_character('G7', 0x78) # G7: "t"
    show_one_character('G8', 0x1C) # G8: "u"
    show_one_character('G9', 0x50) # G9: "r"
    show_one_character('GX', 0x54) # GX: "n"

    show_one_character('H1', 0xB0) # H1: thin vertical double line
    show_one_character('H8', 0x3F) # H8: "O"
    show_one_character('H9', 0x71) # H9: "F"
    show_one_character('HX', 0x71) # HX: "F"

    call_string = 'sudo shutdown -h now'
    os.system(call_string)

    if sbvar.debug_mode != 0:
        exec_time = (datetime.datetime.now() - start_time)
        print(sbvar.col_fe, '    FUNC_END:', sbvar.col_n, 'shutdown_scoreboard()', sbvar.col_t, exec_time, sbvar.col_n)
  
    return None

# END OF FILE